function SMS_ConfirmationScreen( timeoutServer, initialState, completionOperation )
{
    try
    {
        this.m_state = null;
        this.m_stateEntranceTick = null;

        this.m_timeoutServer = timeoutServer;
        this.m_stateDelayTimeoutId = 0;

        this.m_promotionalContent = document.getElementById( 'WIDGET_promotionalContent' );
        this.m_confirmationScreen = document.getElementById( 'WIDGET_confirmationScreen' );

        this.m_hasPromotionalContent = false;

        // set the initial state
        this.ChangeState( initialState, function( ) { } );

        // do the animation
        this.AnimateDown( completionOperation );
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::SMS_ConfirmationScreen( )" );
    }
}

SMS_ConfirmationScreen.prototype=
{
    STATE_SCREEN_ERROR_VIEW         : 0,
    STATE_SCREEN_SUCCESS_VIEW       : 1,
    STATE_SCREEN_IN_PROGRESS_VIEW   : 2,

    MIN_IN_PROGRESS_VIEW_DISPLAY_MS : 2000,

    ERROR_VIEW_BACKGROUND_COLOR     : '#880000',
    SUCCESS_VIEW_BACKGROUND_COLOR   : '#303030',

    BASE_SCREEN_HEIGHT              : 130,

    ANIMATION_STEP                  : 20,
    ANIMATION_INTERVAL_MS           : 10,
    SCREEN_TOP_OFFSET               : 42,

    MAX_SCREEN_WIDTH                : 383,

    /********************************************************
    PUBLIC METHODS
    ********************************************************/

    AnimateUp : function( completionOperation )
    {
        try
        {
            var clipProperties = new this.ClipProperties( this.m_confirmationScreen );

            var self = this;

            var targetTopOffset = parseInt( this.m_confirmationScreen.style.top ) - ( parseInt( this.m_confirmationScreen.style.bottom ) - self.SCREEN_TOP_OFFSET );

            function AnimationStep( )
            {
                if ( parseInt( self.m_confirmationScreen.style.top ) <= targetTopOffset ||
                     parseInt( self.m_confirmationScreen.style.top ) - self.ANIMATION_STEP == targetTopOffset )
                {
                    clipProperties.top = 0;

                    self.m_confirmationScreen.style.display = 'none';

                    self.m_confirmationScreen.style.clip = clipProperties.GetCSSString( );

                    completionOperation( );
                }
                else
                {
                    var moveOffset = Math.min( parseInt( self.m_confirmationScreen.style.top ) - targetTopOffset, self.ANIMATION_STEP );

                    self.MoveUpBy( moveOffset );

                    setTimeout( AnimationStep, self.ANIMATION_INTERVAL_MS );
                }
            }

            AnimationStep( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::AnimateUp( )" );
        }
    },

    MoveUpBy : function( delta )
    {
        try
        {
            // update the top offset
            this.m_confirmationScreen.style.top  = parseInt( this.m_confirmationScreen.style.top ) - delta;

            // update the clip properties
            var clipProperties = new this.ClipProperties( this.m_confirmationScreen );
            clipProperties.top += delta;

            this.m_confirmationScreen.style.clip = clipProperties.GetCSSString( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::MoveUpBy( )" );
        }
    },

    SetHeight : function( height )
    {
        try
        {
            // update the clip properties
            var clipProperties = new this.ClipProperties( this.m_confirmationScreen );
            clipProperties.bottom = height;

            this.m_confirmationScreen.style.clip = clipProperties.GetCSSString( );
            this.m_confirmationScreen.style.height = height;

            document.getElementById( 'WIDGET_confirmationScreenTransparencyBackdrop' ).style.height = height;
            document.getElementById( 'WIDGET_confirmationScreenLayer3' ).style.height = height;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::SetHeight( )" );
        }
    },

    SetPromotionalContent : function( width, content )
    {
        try
        {
            if ( this.m_hasPromotionalContent )
            {
                return;
            }

            this.m_promotionalContent.style.width = ( width > this.MAX_SCREEN_WIDTH ? this.MAX_SCREEN_WIDTH : width );
            this.m_promotionalContent.innerHTML   = content;

            this.m_hasPromotionalContent = true;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::SetPromotionalContent( )" );
        }
    },

    SetErrorMessage : function( message, completionOperation )
    {
        try
        {
            var changeStateCompletionOperation = function( )
            {
                var statusArea = document.getElementById( 'WIDGET_smsSendErrorStatusArea' );
                statusArea.innerHTML = message;

                completionOperation( );
            };

            this.ChangeState( this.STATE_SCREEN_ERROR_VIEW, changeStateCompletionOperation );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::SetErrorMessage( )" );
        }
    },

    SetSuccessMessage : function( message, completionOperation )
    {
        try
        {
            var changeStateCompletionOperation = function( )
            {
                var statusArea = document.getElementById( 'WIDGET_smsSendSuccessStatusArea' );
                statusArea.innerHTML = message;

                completionOperation( );
            };

            this.ChangeState( this.STATE_SCREEN_SUCCESS_VIEW, changeStateCompletionOperation );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::SetSuccessMessage( )" );
        }
    },

    Destructor : function( )
    {
        try
        {
            this.m_timeoutServer.CancelTimeout( this.m_stateDelayTimeoutId );

            this.SetHeight( this.BASE_SCREEN_HEIGHT );

            this.ClearPromotionalContent( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::destructor( )" );
        }
    },

    /********************************************************
    PRIVATE METHODS
    ********************************************************/

    ClipProperties : function( obj )
    {
        try
        {
            var clipStyle = '' + obj.style.clip;

            var matches = /^rect\(\s*([0-9]+)px[, ]\s*([0-9]+)px[, ]\s*([0-9]+)px[, ]\s*([0-9]+)px\s*\)/i.exec( clipStyle );

            this.top    = parseInt( matches[1] );
            this.right  = parseInt( matches[2] );
            this.bottom = parseInt( matches[3] );
            this.left   = parseInt( matches[4] );

            this.GetCSSString = function( )
            {
                try
                {
                    return "rect( " + this.top + "px, " + this.right + "px, " + this.bottom + "px, " + this.left + "px)";
                }
                catch ( e )
                {
                    EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::ClipProperties::GetCSSString( )" );
                }
            };
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::ClipProperties::ClipProperties( )" );
        }
    },

    AnimateDown : function( completionOperation )
    {
        try
        {
            var clipProperties = new this.ClipProperties( this.m_confirmationScreen );
            clipProperties.top = clipProperties.bottom;

            var self = this;
            function AnimationStep( )
            {
                if ( parseInt( self.m_confirmationScreen.style.top ) + self.ANIMATION_STEP >= self.SCREEN_TOP_OFFSET )
                {
                    clipProperties.top = 0;

                    self.m_confirmationScreen.style.top  = self.SCREEN_TOP_OFFSET;
                    self.m_confirmationScreen.style.clip = clipProperties.GetCSSString( );

                    completionOperation( );
                }
                else
                {
                    self.m_confirmationScreen.style.top  = parseInt( self.m_confirmationScreen.style.top ) + self.ANIMATION_STEP;
                    self.m_confirmationScreen.style.clip = clipProperties.GetCSSString( );

                    clipProperties.top -= self.ANIMATION_STEP;

                    setTimeout( AnimationStep, self.ANIMATION_INTERVAL_MS );
                }
            }

            this.m_confirmationScreen.style.top     = this.SCREEN_TOP_OFFSET - parseInt( this.m_confirmationScreen.style.height );
            this.m_confirmationScreen.style.clip    = clipProperties.GetCSSString( );
            this.m_confirmationScreen.style.display = 'block';

            clipProperties.top -= this.ANIMATION_STEP;

            AnimationStep( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::AnimateDown( )" );
        }
    },

    ClearPromotionalContent : function( )
    {
        try
        {
            this.m_promotionalContent.style.width = 0;
            this.m_promotionalContent.innerHTML   = '';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::ClearPromotionalContent( )" );
        }
    },

    ChangeState : function( newState, completionOperation )
    {
        try
        {
            if ( newState === this.m_state )
            {
                completionOperation( );

                return;
            }

            var showView = null;
            var self = this;

            switch ( newState )
            {
            case this.STATE_SCREEN_ERROR_VIEW:
                {
                    showView = function( )
                    {
                        var screenTransparencyBackdrop = document.getElementById( 'WIDGET_confirmationScreenStatusAreaTransparencyBackdrop' );
                        screenTransparencyBackdrop.style.backgroundColor = self.ERROR_VIEW_BACKGROUND_COLOR;

                        document.getElementById( 'WIDGET_smsSendSuccessView' ).style.display = 'none';
                        document.getElementById( 'WIDGET_smsSendErrorView' ).style.display = 'block';
                        document.getElementById( 'WIDGET_smsSendInProgressView' ).style.display = 'none';
                    }
                }
                break;

            case this.STATE_SCREEN_SUCCESS_VIEW:
                {
                    showView = function( )
                    {
                        var screenTransparencyBackdrop = document.getElementById( 'WIDGET_confirmationScreenStatusAreaTransparencyBackdrop' );
                        screenTransparencyBackdrop.style.backgroundColor = self.SUCCESS_VIEW_BACKGROUND_COLOR;

                        document.getElementById( 'WIDGET_smsSendSuccessView' ).style.display = 'block';
                        document.getElementById( 'WIDGET_smsSendErrorView' ).style.display = 'none';
                        document.getElementById( 'WIDGET_smsSendInProgressView' ).style.display = 'none';
                    };
                }
                break;

            case this.STATE_SCREEN_IN_PROGRESS_VIEW:
                {
                    showView = function( )
                    {
                        var screenTransparencyBackdrop = document.getElementById( 'WIDGET_confirmationScreenStatusAreaTransparencyBackdrop' );
                        screenTransparencyBackdrop.style.backgroundColor = self.SUCCESS_VIEW_BACKGROUND_COLOR;

                        document.getElementById( 'WIDGET_smsSendSuccessView' ).style.display = 'none';
                        document.getElementById( 'WIDGET_smsSendErrorView' ).style.display = 'none';
                        document.getElementById( 'WIDGET_smsSendInProgressView' ).style.display = 'block';
                    };
                }
                break;

            default:
                throw new EX_ProgrammingError( "Invalid state: " + newState );
            }

            function ChangeStateImpl( )
            {
                showView( );

                self.m_stateEntranceTick = new Date( ).getTime( );
                self.m_state = newState;

                completionOperation( );
            }

            var currentTick = new Date( ).getTime( );

            if ( this.m_state !== this.STATE_SCREEN_IN_PROGRESS_VIEW ||
                 this.m_stateEntranceTick === null                   ||
                 currentTick - this.m_stateEntranceTick >= this.MIN_IN_PROGRESS_VIEW_DISPLAY_MS )
            {
                ChangeStateImpl( );
            }
            else
            {
                this.m_stateDelayTimeoutId =
                    this.m_timeoutServer.RequestTimeout( ChangeStateImpl,
                                                         this.MIN_IN_PROGRESS_VIEW_DISPLAY_MS -
                                                         ( currentTick - this.m_stateEntranceTick ) );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_ConfirmationScreen::ChangeState( )" );
        }
    }
};
