/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/


/******************************************************************************
  SMS_RegisterView c'tor

    HTML DOM Dependencies:
        WIDGET_registerViewFrame          - <div>
        WIDGET_registerViewStatusTextArea - <div>
        WIDGET_registerCellNumberTextArea - <input type="text">
        WIDGET_registerEmailTextArea      - <input type="text">
        WIDGET_registerPinTextArea        - <input type="password">

    Call backs:
        onRegisterAccountSuccess( responseMessage )

    Globals:
        VM_GetPreference( key )
        VM_SetPreference( key, value )
        EX_Log( message )
        VM_WIDGET_SOURCE
        VM_WIDGET_VERSION
        VM_WIDGET_OS_VERSION

 ******************************************************************************/
function SMS_RegisterView( mobileNumberControlSynchronizer, onRegisterAccountSuccess )
{
    try
    {
        this.m_onRegisterAccountSuccess = onRegisterAccountSuccess;

        // Account handler
        var self = this;
        this.m_accountHandler = new ACC_AccountHandler( VM_WIDGET_SOURCE,
                                                        VM_WIDGET_VERSION,
                                                        VM_WIDGET_OS_VERSION );
        this.m_accountHandler.m_onRegisterAccountSucceeded = function( responseMessage )
        {
            self.onRegisterAccountSucceeded( responseMessage );
        };
        this.m_accountHandler.m_onRegisterAccountFailed    = function( statusMessage )
        {
            self.onRegisterAccountFailed( statusMessage );
        };

        function AddEditFieldMethod( self, identifier )
        {
            self[ 'Get' + identifier + 'Obj' ] = function( message )
            {
                return document.getElementById( 'WIDGET_register' + identifier + 'TextArea' );
            };

            self[ 'Get' + identifier + 'Data' ] = function( message )
            {
                return self[ 'Get' + identifier + 'Obj' ]( ).value;
            };

            self[ 'Set' + identifier + 'Data' ] = function( message )
            {
                self[ 'Get' + identifier + 'Obj' ]( ).value = message;
            };
        }

        AddEditFieldMethod( this, 'CustName' );
        AddEditFieldMethod( this, 'Email' );
        AddEditFieldMethod( this, 'Pin' );

        this.m_mobileNumberControl = new SMS_MobileNumberControl( 'WIDGET_registerCountryPopup',
                                                                  'WIDGET_registerCellNumberTextArea' );

        mobileNumberControlSynchronizer.AddControl( this.m_mobileNumberControl );
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, "SMS_RegisterView::SMS_RegisterView( )" );
    }
}
SMS_RegisterView.prototype =
{
    CUST_NAME_MESSAGE         : "Please enter a valid name.",
    DEFAULT_REG_ERROR_MESSAGE : "Registration failed. Please try again later.",
    DEFAULT_MSG_COLOR         : "#000000",
    ERROR_MSG_COLOR           : "#bf0000",

    /**************************************************************************
      Accessor and modify methods
     **************************************************************************/

    SetStatusText : function ( message, color )
    {
        try
        {
            UTILS.SetInnerHTML( 'WIDGET_registerViewStatusTextArea', message, color );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_RegisterView::SetStatusText( )" );
        }
    },

    GetDimensions : function( )
    {
        var obj = document.getElementById( 'WIDGET_registerViewFrame' );
        return { width: obj.clientWidth, height: obj.clientHeight };
    },

    /**************************************************************************
      View repainting methods
     **************************************************************************/

    reset:function( )
    {
        try
        {
            document.getElementById( 'WIDGET_registerViewFrame' ).style.display = 'none';

            this.SetPinData( "" );

            this.m_accountHandler.StopRegistrationRequests( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_RegisterView::reset( )" );
        }
    },

    refresh:function( )
    {
        try
        {
            document.getElementById( 'WIDGET_registerViewFrame' ).style.display = 'block';

            this.SetStatusText( "Please enter the following information to register:", this.DEFAULT_MSG_COLOR );

            var email = VM_GetPreference( PREF_KEY_SENDER_EMAIL );

            if ( email )
            {
                this.SetEmailData( email );
            }

            var custName = VM_GetPreference( PREF_KEY_SENDER_NAME );

            if ( custName )
            {
                this.SetCustNameData( custName );
            }

            this.GetCustNameObj( ).focus( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_RegisterView::refresh( )" );
        }
    },

    /**************************************************************************
      Register methods
     **************************************************************************/

    handleRegistrationSubmit:function( )
    {
        try
        {
            if ( this.m_accountHandler.IsRegistrationRequestInProgress( ) )
            {
                // ignore
                return;
            }

            var custName = null;

            try
            {
                custName = new USER_SubscriberName( this.GetCustNameData( ) );
            }
            catch ( e )
            {
                this.SetStatusText( this.CUST_NAME_MESSAGE, this.ERROR_MSG_COLOR );
                return;
            }

            this.m_mobileNumberControl.OnFormSubmit( );

            var numberValidationData = this.m_mobileNumberControl.GetValidatedNumber( );

            if ( numberValidationData.ERROR_MESSAGE )
            {
                this.SetStatusText( numberValidationData.ERROR_MESSAGE, this.ERROR_MSG_COLOR );
                return;
            }

            var PIN = UTILS_StripNonDigits( this.GetPinData( ) );

            if ( PIN.length != this.GetPinData( ).length )
            {
                this.SetPinData( '' );
                this.SetStatusText( "Please enter a 4-10 digit PIN (numeric characters only).", this.ERROR_MSG_COLOR );
                return;
            }

            if ( PIN.length < 4 || PIN.length > 10 )
            {
                this.SetPinData( '' );
                this.SetStatusText( "Please enter a 4-10 digit PIN.", this.ERROR_MSG_COLOR );
                return;
            }

            var email = this.GetEmailData( );

            if ( email.length == 0 || !SMS_UTILS_IsValidEmailAddress( email ) )
            {
                this.SetStatusText( "Please enter a valid email address.", this.ERROR_MSG_COLOR );
                return;
            }

            VM_SetPreference( PREF_KEY_SENDER_MOBILE, numberValidationData.PHONE_NUMBER_STATE.AsString( ) );

            this.SetStatusText( "Connecting, please wait...", this.DEFAULT_MSG_COLOR );

            this.m_accountHandler.registerAccount( numberValidationData.PHONE_NUMBER.GetFullyQualifiedString( ),
                                                   PIN,
                                                   email,
                                                   custName.name );
        }
        catch ( e )
        {
            this.SetStatusText( this.DEFAULT_REG_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

            EX_Log( "SMS_RegisterView::handleRegistrationSubmit( ) : " + e.message );
        }
    },

    onRegisterAccountSucceeded:function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    /*
                    TODO: what to do with responseMessage.m_successCase attribute
                    */

                    // Save persistent account attributes
                    VM_SetPreference( PREF_KEY_ACCOUNT, responseMessage.m_account );
                    VM_SetPreference( PREF_KEY_U_STRING, responseMessage.m_uString );

                    this.m_onRegisterAccountSuccess( );
                    break;
                }

            case MSG_ResponseStatus.UNEXPECTED_ERROR:
            case MSG_ResponseStatus.RECOVERABLE_ERROR:
                {
                    this.SetStatusText( this.DEFAULT_REG_ERROR_MESSAGE, this.ERROR_MSG_COLOR );
                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
                {
                    this.SetStatusText( responseMessage.m_statusMsg, this.ERROR_MSG_COLOR );
                    break;
                }

            case MSG_ResponseStatus.REGA_PIN_MISMATCH:
                {
                    SMS_widgetViewController.ShowLoginView( );
                    SMS_widgetViewController.GetLoginView( ).SetStatusText(
                        responseMessage.m_statusMsg ? responseMessage.m_statusMsg : '',
                        SMS_widgetViewController.GetLoginView( ).ERROR_MSG_COLOR );

                    break;
                }

            default:
                {
                    if ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 )
                    {
                        this.SetStatusText( responseMessage.m_statusMsg, this.ERROR_MSG_COLOR );
                    }
                    else
                    {
                        this.SetStatusText( this.DEFAULT_REG_ERROR_MESSAGE, this.ERROR_MSG_COLOR );
                    }
                    break;
                }
            }
        }
        catch ( e )
        {
            EX_Log( "SMS_RegisterView::onRegisterAccountSucceeded( ) : " + e );

            this.SetStatusText( this.DEFAULT_REG_ERROR_MESSAGE, this.ERROR_MSG_COLOR );
        }
    },

    onRegisterAccountFailed:function( statusMessage )
    {
        try
        {
            this.SetStatusText( this.DEFAULT_REG_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

            EX_Log( "SMS_RegisterView::onRegisterAccountFailed( ) : " + statusMessage );
        }
        catch ( e )
        {
            EX_Log( "SMS_RegisterView::onRegisterAccountFailed( ) : " + e );
        }
    }
};